using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Xml.Serialization;

namespace EQIA.EQIDL
{
    /// <summary>
    /// Template used when drawing a button, stores a reference to animations used in the button, and provides a draw function.
    /// </summary>
    [TypeConverter(typeof(ButtonDrawTemplateConverter))]
    public partial class ButtonDrawTemplate : Class
    {
        #region Members

        private string myNormal, myPressed, myFlyby, myDisabled, myPressedFlyby;
        private string myNormalDecal, myPressedDecal, myFlybyDecal, myDisabledDecal, myPressedFlybyDecal;

        #endregion

        #region Properties

        /// <summary>
        /// Name of the animation to use for a button without any interaction.
        /// </summary>
        [XmlElement("Normal", IsNullable = false)]
        [Browsable(true), Description("Name of the animation to use for a button without any interaction.")]
        [NotifyParentProperty(true)]
        public string Normal { get { return myNormal; } set { myNormal = value; } }

        /// <summary>
        /// Name of the animation to use for a button that is pressed down without the mouse hovering over it.
        /// </summary>
        [XmlElement("Pressed", IsNullable = false)]
        [Browsable(true), Description("Name of the animation to use for a button that is pressed down without the mouse hovering over it.")]
        [NotifyParentProperty(true)]
        public string Pressed { get { return myPressed; } set { myPressed = value; } }

        /// <summary>
        /// Name of the animation to use for a button that the mouse hovering over.
        /// </summary>
        [XmlElement("Flyby", IsNullable = false)]
        [Browsable(true), Description("Name of the animation to use for a button that the mouse hovering over.")]
        [NotifyParentProperty(true)]
        public string Flyby { get { return myFlyby; } set { myFlyby = value; } }

        /// <summary>
        /// Name of the animation to use for a button that has been disabled.
        /// </summary>
        [XmlElement("Disabled", IsNullable = false)]
        [Browsable(true), Description("Name of the animation to use for a button that has been disabled.")]
        [NotifyParentProperty(true)]
        public string Disabled { get { return myDisabled; } set { myDisabled = value; } }

        /// <summary>
        /// Name of the animation to use for a button that is pressed down with the mouse hovering over it.
        /// </summary>
        [XmlElement("PressedFlyby", IsNullable = false)]
        [Browsable(true), Description("Name of the animation to use for a button that is pressed down with the mouse hovering over it.")]
        [NotifyParentProperty(true)]
        public string PressedFlyby { get { return myPressedFlyby; } set { myPressedFlyby = value; } }

        /// <summary>
        /// Name of the animation to draw over top of a normal button.
        /// </summary>
        [XmlElement("NormalDecal", IsNullable = false)]
        [Browsable(true), Description("Name of the animation to draw over top of a normal button.")]
        [NotifyParentProperty(true)]
        public string NormalDecal { get { return myNormalDecal; } set { myNormalDecal = value; } }

        /// <summary>
        /// Name of the animation to draw over top of a pressed button (will use normal decal if it isn't set).
        /// </summary>
        [XmlElement("PressedDecal", IsNullable = false)]
        [Browsable(true), Description("Name of the animation to draw over top of a pressed button (will use normal decal if it isn't set).")]
        [NotifyParentProperty(true)]
        public string PressedDecal { get { return myPressedDecal; } set { myPressedDecal = value; } }

        /// <summary>
        /// Name of the animation to draw over top of a button with the mouse hovering over it (will use normal decal if it isn't set).
        /// </summary>
        [XmlElement("FlybyDecal", IsNullable = false)]
        [Browsable(true), Description("Name of the animation to draw over top of a button with the mouse hovering over it (will use normal decal if it isn't set).")]
        [NotifyParentProperty(true)]
        public string FlybyDecal { get { return myFlybyDecal; } set { myFlybyDecal = value; } }

        /// <summary>
        /// Name of the animation to draw over top of a disabled button (will use normal decal if it isn't set).
        /// </summary>
        [XmlElement("DisabledDecal", IsNullable = false)]
        [Browsable(true), Description("Name of the animation to draw over top of a disabled button (will use normal decal if it isn't set).")]
        [NotifyParentProperty(true)]
        public string DisabledDecal { get { return myDisabledDecal; } set { myDisabledDecal = value; } }

        /// <summary>
        /// Name of the animation to draw over top of a pressed button with the mouse hovering over it (will use normal decal if it isn't set).
        /// </summary>
        [XmlElement("PressedFlybyDecal", IsNullable = false)]
        [Browsable(true), Description("Name of the animation to draw over top of a pressed button with the mouse hovering over it (will use normal decal if it isn't set).")]
        [NotifyParentProperty(true)]
        public string PressedFlybyDecal { get { return myPressedFlybyDecal; } set { myPressedFlybyDecal = value; } }

        #endregion

        #region Creation

        public ButtonDrawTemplate()
            : this(null)
        {
        }

        public ButtonDrawTemplate(EQInterface eqInterface)
            : base(eqInterface)
        {
        }

        #endregion

        #region Deserialization

        protected override void SetProperties(object obj)
        {
            base.SetProperties(obj);

            ButtonDrawTemplate buttonDrawTemplate = obj as ButtonDrawTemplate;
            if (buttonDrawTemplate != null)
            {
                Normal = buttonDrawTemplate.Normal;
                NormalDecal = buttonDrawTemplate.NormalDecal;
                Pressed = buttonDrawTemplate.Pressed;
                PressedDecal = buttonDrawTemplate.PressedDecal;
                Flyby = buttonDrawTemplate.Flyby;
                FlybyDecal = buttonDrawTemplate.FlybyDecal;
                Disabled = buttonDrawTemplate.Disabled;
                DisabledDecal = buttonDrawTemplate.DisabledDecal;
                PressedFlyby = buttonDrawTemplate.PressedFlyby;
                PressedFlybyDecal = buttonDrawTemplate.PressedFlybyDecal;
            }
        }

        #endregion
    }

    public class ButtonDrawTemplateConverter : ExpandableObjectConverter
    {
        public override object ConvertTo(ITypeDescriptorContext context, System.Globalization.CultureInfo culture, object value, System.Type destinationType)
        {
            return (destinationType == typeof(string)) ? string.Empty : base.ConvertTo(context, culture, value, destinationType);
        }
    }
}
